/**
* @author Vincent Pippan
* @version 1.0
* Projekt: Routenstatistik von Segelregatten
* Datei: ProgrammRun.java
* Last Updated: 26. 5. 2002
*Beschreibung: Klasse zur Programmausführung
*/

import java.io.*;
	
public class ProgrammRun
{	
	private String filelist[], inline, out;
	private int index = 0, timeindex = 0;
	
	InputStringParse parse = new InputStringParse();
	VectorFunctions vector = new VectorFunctions();
	XYPosition position = new XYPosition();
	
	/** public void Run Programm(String path, String laposref, String loposref, masstab)
	* @param path Pfad zu den Sourcedateien
	* @param laposref Latitude Positions Referenz (Y - Nullpunkt)
	* @param loposref Longitude Positions Referenz (X - Nullpunkt)
	* @param masstab Anzahl der Pixel pro Meter auf der Computerkarte
	*/
	public void RunProgramm(String path, String laposref, String loposref, int masstab)
	{
		//Create a file object from a directory
		File datasource = new File(path);
		File datadestination = new File(path+"\\visualisation_data");
		
		//Write all filenames from a directory to a string array
		filelist = datasource.list();
		
		//Create a new Directory for the output data
		datadestination.mkdir();
		
		//Do the same operations for each file in a directory
		while(index != filelist.length)
		{
			try
			{
				//Create a LineNumberReader for a file from filelist
				LineNumberReader fin = new LineNumberReader(new FileReader(path+"\\"+filelist[index]));
				
				//Create a new output file
				File datadestinationfile = new File(datadestination+"\\"+filelist[index]+"_output.vda");
				datadestinationfile.createNewFile();
				FileWriter fout = new FileWriter(datadestinationfile);
				
				//Read all lines from a file
				while ((inline = fin.readLine()) != null) 
				{
					if(timeindex > 3) 
					{
						//Parse a line from the file
						parse.StringParse(inline);
						
						//Set the reference point
						position.XYInit(laposref,loposref);
						
						//Calculate the data
						vector.BoatData(parse.nsvd,parse.ewvd,parse.nsvm,parse.ewvm);
						vector.WindData(parse.wd,parse.wvm);
						position.CalculateXY(masstab,parse.lap,parse.lop);
						
						//Write to the output file
						fout.write("@ x"+position.x+" Y"+position.y+" a"+vector.realwd+
								   " b"+vector.realwvm+" c"+vector.relwd+" d"+vector.relwvm+
								   " e"+vector.boatspeed+" f"+vector.boatdirection+
								   " g"+timeindex+" h"+parse.hper+" s"+parse.ps+"\r\n");
					}
					timeindex = timeindex + 1;
				}
				
				//Write Data Termination Character
				fout.write("ElIf Fo DnE");
				
				fin.close();
				fout.close();
				timeindex = 0;
				System.out.println("File: "+filelist[index]+" bearbeitet!\n");
			}
			
			//Error Message
			catch (Exception e) 
			{
				System.out.println("Fehler beim bearbeiten der Datei: "+filelist[index]+"\nFehler: "+e+"\n");
			}
			index = index + 1;
		}
	}
}