/**
* @author Vincent Pippan
* @version 1.0
* Projekt: Routenstatistik von Segelregatten
* Datei: VectorFunction.java
* Last Updated: 26. 5. 2002
* Beschreibung: Funktionen zur Berechnung der Wind- und Bootsdaten
*/

import java.lang.Math;
import java.lang.Double;


public class VectorFunctions
{
	public String wd, wvm;
	public double boatspeed, boatdirection, realwd, realwvm;
	public double relwd, relwvm; 
	
	private double dewvm, dnsvm;
	private double xwind, ywind, xrealwind, yrealwind, xboat, yboat;
	
	/**Calculate boatspeed and boatdirection
	* This function has to be called before WindData()!!!
	* public void BoatData(char nsvd, char ewvd, String nsvm, String ewvm
	* @param nsvd N/S velocity direction
	* @param ewvd E/W velocity direction
	* @param nsvm N/S velocity magnitude
	* @param ewvm E/W velocity magnitude
	*/
	public void BoatData(char nsvd, char ewvd, String nsvm, String ewvm)
	{
		//Convert the Strings to the correct data types
		dewvm = (Double.parseDouble(ewvm) / 10);	//Divide by 10, because 1234 from the string means 123.4
		dnsvm = (Double.parseDouble(nsvm) / 10);
		
		//Calculate boatspeed and boatdirection (only in Sector 1 of the coordinate system)
		boatspeed = Math.sqrt((dewvm*dewvm)+(dnsvm*dnsvm));	//Calculate the boatspeed
		boatdirection = Math.toDegrees(Math.atan(dewvm/dnsvm));	//Calculate the course in degree (0 - 90°)
		
		//Check for the Sector, where the vector should be, and calculate the correct angel
		//Compass Rose: N=0°, E=90°, S=180°, W=270°
		//if(nsvd=='N' && ewvd=='E') boatdirection = boatdirection;		//Calculated max. values without correction: N=0°, E=90° 
		if(nsvd=='N' && ewvd=='W') boatdirection = 360 - boatdirection;	//Calculated max. values without correction: N=0°, W=90°
		if(nsvd=='S' && ewvd=='W') boatdirection = 180 + boatdirection;	//Calculated max. values without correction: S=0°, W=90°
		if(nsvd=='S' && ewvd=='E') boatdirection = 180 - boatdirection;	//Calculated max. values without correction: S=0°, E=90°
	
		//Check for division through NULL
		if(Double.isNaN(boatdirection)) boatdirection = 0;
	}
	
	/**Calculate WindData
	* public void WindData(String wd, String wvm)
	* @param wd Windrichtung
	* @param wvm Wind velocity magnitude
	*/
	public void WindData(String wd, String wvm)
	{
		//Convert the strings to the correct data types
		relwd = Integer.parseInt(wd);
		relwvm = (Double.parseDouble(wvm) / 10);	//Divide by 10, because 123 from the string means 12.3
		
		//wd is the difference between the relativ wind direction, and the boat course -> calculate the real angel
		relwd = Math.abs(relwd - (360 - boatdirection));
		
		//Calculate the component form of the vectors
		xwind = relwvm * Math.cos(Math.toRadians(relwd));
		ywind = relwvm * Math.sin(Math.toRadians(relwd));
		xboat = boatspeed * Math.cos(Math.toRadians(boatdirection));
		yboat = boatspeed * Math.sin(Math.toRadians(boatdirection));
		
		//Because the compass rose rotates clockworth (coordinate system rotates counter clockworth), correct the y-values
 		ywind = ywind * (-1);
		yboat = yboat * (-1);
		
		//Calculate the real wind
		xrealwind = xwind + xboat;	//Because the driving wind, is the opossite of the boatspeed and boatdirection,
		yrealwind = ywind + yboat;	//add the winddata and the boatdata
		
		//Transfer back to polar form
		realwvm = Math.sqrt((xrealwind*xrealwind)+(yrealwind*yrealwind));	//Calculate the real windspeed
		realwd = Math.toDegrees(Math.atan(yrealwind/xrealwind)); //Calculate the real winddirection in degree (-90° to 90°)
		
		//Check for the sector, where the vector should be, and calculate the correct angel
		if(xrealwind>=0 && yrealwind<=0) realwd = realwd * (-1);	//old realwd negativ
		if(xrealwind<=0 && yrealwind<=0) realwd = 180 - realwd;		//old realwd positive
		if(xrealwind<=0 && yrealwind>=0) realwd = 180 - realwd;		//old realwd negativ
		if(xrealwind>=0 && yrealwind>=0) realwd = 360 - realwd;		//old realwd positiv
	
		//Check for division through NULL
		if(Double.isNaN(realwd))	realwd = 0;
		if(Double.isNaN(realwvm)) realwvm = 0;
		if(Double.isNaN(relwd)) relwd = 0;
		
		//Bei einer Bootsgeschwindigkeit von Null, wird ein falscher Wert ausgegeben.
	}
}