/**
* @author Vincent Pippan
* @version 1.0
* Projekt: Routenstatistik von Segelregatten
* Datei: XYPosition.java
* Last Updated: 26. 5. 2002
* Beschreibung: Funktionen zur Berechnung der X und Y Werte zur grafischen Darstellung der Bootsbewegungen
*/

public class XYPosition
{
	public int x, y;
	
	private int ladeg, lodeg, laposrefdeg, loposrefdeg, ladifdeg, lodifdeg;
	private double lamin, lomin, laposrefmin, loposrefmin, ladifmin, lodifmin;
	private char laposrefh, loposrefh;
	
	/**Set the first point for the calculation
	* public void XYInit(String laposref, String loposref)
	* @param laposref Latitude Positionsreferenz (Y - Nullpunkt)
	* @param loposref Longitude Positionsreferenz (X - Nullpunkt)
	*/
	public void XYInit(String laposref, String loposref)
	{
		//Parse the strings, and convert them to the correct data types
		//Position Referenz
		laposrefh = (laposref.substring(0,1)).charAt(0);
		laposrefdeg = Integer.parseInt(laposref.substring(1,3));
		laposrefmin = (Double.parseDouble(laposref.substring(3)) / 1000);
		loposrefh = (loposref.substring(0,1)).charAt(0);
		loposrefdeg = Integer.parseInt(loposref.substring(1,4));
		loposrefmin = (Double.parseDouble(loposref.substring(4)) / 1000);
	}
	
	/**Calculate the x and y values
	* public void CalculateXY(int masstab, String lap, String lop)
	* @param masstab Anzahl der Pixel pro Meter auf der Computerkarte
	* @param lap Latitude Position Data
	* @param lop Longitude Position Data
	*/
	public void CalculateXY(int masstab, String lap, String lop)
	{
		//This function does not look at the hemisphere
		//So if there are different hemispheres there may be problems or false XY values
		
		//Parse the strings, and convert them to the correct data types
		//Latitude
		ladeg = Integer.parseInt(lap.substring(0,2));
		lamin = (Double.parseDouble(lap.substring(2)) / 1000);	//Divide by 1000, because 12345 from the string means 12.345
		//Longitude
		lodeg = Integer.parseInt(lop.substring(0,3));
		lomin = (Double.parseDouble(lop.substring(3)) / 1000);
		
		//1sm = 1,852km = 1852m
		//1ä = 60sm = 111.12km = 111120m
		
		//Calculate the difference between two points
		ladifdeg = laposrefdeg - ladeg;
		lodifdeg = loposrefdeg - lodeg;
		ladifmin = laposrefmin - lamin;
		lodifmin = loposrefmin - lomin;
			
		//Store the last used points as new position reference
		laposrefdeg = ladeg;
		laposrefmin = lamin;
		loposrefdeg = lodeg;
		loposrefmin = lomin;
		
		//Convert the difference from degree to meters
		ladifdeg = ladifdeg * (111120);
		lodifdeg = lodifdeg * (111120);
		
		//Convert the difference from minutes to meters
		ladifmin = ladifmin * 1852;
		lodifmin = lodifmin * 1852;
		
		//Calculate the XY Values
		x = (int) ((lodifdeg + lodifmin) / masstab);
		y = (int) ((ladifdeg + ladifmin) / masstab);
	}
}